; Babbage difference engine calculation of 4.arctan(x)
;  over the interval 0.0..1.0 using 10th order polynomial
;  approximation and 16 digit differences.
;
; 4.arctan(x) is calculated for fifty values over the
;  interval 0.0..1.0, the final reading of which should
;  be 4.arctan(1.0) = Pi.
;
; Runs in unexpanded Mk14
;
; K Orton 2007
;
; Assembled using the very excellent 'AS' assembler by
;  Alfred Arnold et al
;
; Vive la difference!!!!
;
; Sorry, I couldn't resist that
;
; Adapted from PIC14 AS source to sbasm by TJG

	.CR	SCMP
	.TF	BABBAGE.HEX,INT

ORDER	.EQ	10
NITER	.EQ	50
DISP	.EQ	0x0D00
ITCNT	.EQ	0x0F12
CNT1	.EQ	0x0F13
CNT2	.EQ	0x0F14

; Program entry point
	.OR	0x0F20
	
	LDI	DISP>>8			; Point P1 at display
	XPAH	P1
	LDI	SEGTBL>>8		; Point P2 at segment table
	XPAH	P2
	LDI	SEGTBL&0x00ff
	XPAL	P2
	LDI	NITER			; Iteration count
	ST	ITCNT
MAINLP	LDI	DIFTBL+4>>8		; Display current output
	XPAH	P3			;  for a moment
	LDI	DIFTBL+4&0x00ff
	XPAL	P3
	LDI	0
	ST	CNT1	
DSPLP1	LDI	DISP&0x00ff		; Reset display pointer
	XPAL	P1
	LDI	4
	ST	CNT2
DSPLP2	LD	0(P3)			; Lower digit
	ANI	0x0F	
	XAE
	LD	-128(P2)
	ST	@+1(P1)
	LDI	11
	DLY	0
	LD	@+1(P3)			; Upper digit
	SR
	SR
	SR
	SR
	XAE
	LD	-128(P2)
	ST	@+1(P1)
	LDI	0
	DLY	0
	DLD	CNT2
	JNZ	DSPLP2
	LDI	11
	DLY	0
	LDI	0			; Blank display	
	ST	0(P1)
	LD	@-4(P3)			; Move table pointer
	DLD	CNT1			;  back
	JNZ	DSPLP1
	LD	ITCNT
	JZ	MAINLP
	LD	@-4(P3)			; So P3 points at table

; This is actual difference calculation.
;
; What Babbage would have given for this!
;
	LDI	ORDER
	ST	CNT1
DIFLP1	LDI	8
	ST	CNT2
	CCL
DIFLP2	LD	0(P3)
	DAD	8(P3)
	ST	@+1(P3)
	DLD	CNT2
	JNZ	DIFLP2
	DLD	CNT1
	JNZ	DIFLP1
	DLD	ITCNT
	JMP	MAINLP

; Segment table
; There is a copy of this in SCIOS but I've seen several
;  versions of SCIOS so I'm not relying on this table
;  staying still!
;
SEGTBL	.DB	0x3F
	.DB	0x06
	.DB	0x5B
	.DB	0x4F
	.DB	0x66
	.DB	0x6D
	.DB	0x7D
	.DB	0x07
	.DB	0x7F
	.DB	0x67

; The difference table. These rows correspond to the
;  digit columns in the original difference engine.
;
; Note that table is in BCD format and that negative
;  numbers are stored in 'ten's complement' format
;  i.e. -1 is represented as 9999999999999999
;
; This table is modified by the program and will require
;  reloading for another run. Well - you'd have to do
;  the same with a real difference engine!
;
DIFTBL	.DB	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.DB	0x91,0x03,0x44,0x30,0x92,0x98,0x79,0x00
	.DB	0x83,0x49,0x41,0x90,0x61,0x93,0x99,0x99
	.DB	0x24,0x23,0x26,0x00,0x63,0x93,0x99,0x99
	.DB	0x21,0x67,0x69,0x55,0x06,0x00,0x00,0x00
	.DB	0x03,0x60,0x50,0x72,0x02,0x00,0x00,0x00
	.DB	0x91,0x19,0x36,0x98,0x99,0x99,0x99,0x99
	.DB	0x20,0x55,0x86,0x93,0x99,0x99,0x99,0x99
	.DB	0x72,0x79,0x06,0x01,0x00,0x00,0x00,0x00
	.DB	0x99,0x74,0x90,0x99,0x99,0x99,0x99,0x99
	.DB	0x62,0x36,0x00,0x00,0x00,0x00,0x00,0x00

;	END
